@@ -1,21 +1,21 @@
 /*
-  The oRTP library is an RTP (Realtime Transport Protocol - rfc3550) stack.
-  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * Copyright (c) 2016-2020 Belledonne Communications SARL.
+ *
+ * This file is part of bctoolbox.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 /* this file is responsible of the portability of the stack */
 
 #ifndef BCTBX_PORT_H
@@ -36,8 +36,9 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
+#include <wchar.h>
 
-#ifdef __linux
+#ifdef __linux__
 #include <stdint.h>
 #endif
 
@@ -117,6 +118,7 @@ unsigned long __bctbx_thread_self(void);
 
 #define getSocketError() strerror(errno)
 #define getSocketErrorCode() (errno)
+#define getSocketErrorWithCode(code) strerror(code)
 #define bctbx_gettimeofday(tv,tz) gettimeofday(tv,tz)
 #define bctbx_log10f(x)	log10f(x)
 
@@ -139,13 +141,107 @@ unsigned long __bctbx_thread_self(void);
 #if defined(__MINGW32__) || !defined(WINAPI_FAMILY_PARTITION) || !defined(WINAPI_PARTITION_DESKTOP)
 #define BCTBX_WINDOWS_DESKTOP 1
 #elif defined(WINAPI_FAMILY_PARTITION)
+//WINAPI_PARTITION_PC_APP => UWP
+//WINAPI_PARTITION_DESKTOP => Win32 Only
+//WINAPI_PARTITION_PHONE_APP => Windows Phone Store
+//WINAPI_PARTITION_APP => UWP+Windows Phone Store
+// On UWP, we have : 1001
+// On Desktop : we have : 1101
+// Uncomment line below to check combination
+//#define BCTBX_CHECK_FAMILY
+#ifdef BCTBX_CHECK_FAMILY
+        #if defined (WINAPI_PARTITION_PC_APP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_PC_APP)
+        #define FAMILY1
+        #endif
+        #if defined (WINAPI_PARTITION_DESKTOP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+        #define FAMILY2
+        #endif
+        #if defined (WINAPI_PARTITION_PHONE_APP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_PHONE_APP)
+        #define FAMILY3
+        #endif
+        #if defined (WINAPI_PARTITION_APP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP)
+        #define FAMILY4
+        #endif
+
+        #ifdef FAMILY1
+                #ifdef FAMILY2
+                        #ifdef FAMILY3
+                                #ifdef FAMILY4
+                                        #error 1111
+                                #else
+                                        #error 1110
+                                #endif
+                        #else
+                                #ifdef FAMILY4
+                                        #error 1101
+                                #else
+                                        #error 1100
+                                #endif
+
+                        #endif
+                #else
+                        #ifdef FAMILY3
+                                #ifdef FAMILY4
+                                        #error 1011
+                                #else
+                                        #error 1010
+                                #endif
+                        #else
+                                #ifdef FAMILY4
+                                        #error 1001
+                                #else
+                                        #error 1000
+                                #endif
+
+                        #endif
+                #endif
+        #else
+                #ifdef FAMILY2
+                        #ifdef FAMILY3
+                                #ifdef FAMILY4
+                                        #error 0111
+                                #else
+                                        #error 0110
+                                #endif
+                        #else
+                                #ifdef FAMILY4
+                                        #error 0101
+                                #else
+                                        #error 0100
+                                #endif
+
+                        #endif
+                #else
+                        #ifdef FAMILY3
+                                #ifdef FAMILY4
+                                        #error 0011
+                                #else
+                                        #error 0010
+                                #endif
+                        #else
+                                #ifdef FAMILY4
+                                        #error 0001
+                                #else
+                                        #error 0000
+                                #endif
+
+                        #endif
+                #endif
+        #endif
+#endif
+
 #if defined(WINAPI_PARTITION_DESKTOP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
 #define BCTBX_WINDOWS_DESKTOP 1
+#elif defined (WINAPI_PARTITION_PC_APP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_PC_APP)
+#define BCTBX_WINDOWS_DESKTOP 1
+#define BCTBX_WINDOWS_UWP 1
 #elif defined(WINAPI_PARTITION_PHONE_APP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_PHONE_APP)
 #define BCTBX_WINDOWS_PHONE 1
 #elif defined(WINAPI_PARTITION_APP) && WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP)
 #define BCTBX_WINDOWS_UNIVERSAL 1
 #endif
+
+
 #endif
 
 #ifndef BCTBX_DEPRECATED
@@ -275,13 +371,24 @@ const char * bctbx_strerror(DWORD value);
 
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 BCTBX_PUBLIC const char *__bctbx_getWinSocketError(int error);
 
+#ifdef __cplusplus
+}
+#endif
+
 #ifndef getSocketErrorCode
-#define getSocketErrorCode()   WSAGetLastError()
+#define getSocketErrorCode()           WSAGetLastError()
 #endif
 #ifndef getSocketError
-#define getSocketError()       __bctbx_getWinSocketError(WSAGetLastError())
+#define getSocketError()               __bctbx_getWinSocketError(WSAGetLastError())
+#endif
+#ifndef getSocketErrorWithCode
+#define getSocketErrorWithCode(code)   __bctbx_getWinSocketError(code)
 #endif
 
 #ifndef snprintf
@@ -363,29 +470,60 @@ void bctbx_set_memory_functions(BctoolboxMemoryFunctions *functions);
 
 BCTBX_PUBLIC int bctbx_socket_set_non_blocking(bctbx_socket_t sock);
 
+#ifdef __GNUC__
+#	define BCTBX_PRINTF_LIKE(format_pos, args_pos) __attribute__ ((format (printf, format_pos, args_pos)))
+#else
+#	define BCTBX_PRINTF_LIKE(format_pos, args_pos)
+#endif
+
 BCTBX_PUBLIC char *bctbx_strndup(const char *str,int n);
-BCTBX_PUBLIC char *bctbx_strdup_printf(const char *fmt,...);
+BCTBX_PUBLIC char *bctbx_strdup_printf(const char *fmt,...) BCTBX_PRINTF_LIKE(1, 2);
 BCTBX_PUBLIC char *bctbx_strdup_vprintf(const char *fmt, va_list ap);
-BCTBX_PUBLIC char *bctbx_strcat_printf(char *dst, const char *fmt,...);
+BCTBX_PUBLIC char *bctbx_strcat_printf(char *dst, const char *fmt,...) BCTBX_PRINTF_LIKE(2, 3);
 BCTBX_PUBLIC char *bctbx_strcat_vprintf(char *dst, const char *fmt, va_list ap);
-BCTBX_PUBLIC char *bctbx_concat (const char *str, ...) ;
+BCTBX_PUBLIC char *bctbx_concat(const char *str, ...);
+BCTBX_PUBLIC char *bctbx_replace(char *str, char c, char n);
 
 /**
- * Portable version of the dirname function from libgen.h
- * @param[in] path The full path for which we want to find the dirname
- * @return NULL if no dirname is found, otherwise a copy of the dirname of path that needs to be freed with bctbx_free().
+ * @brief Return the directory part of a file path.
+ *
+ * Find the last delimiting character ('/' or '\') and return a
+ * copy of the substring before. If no delimiter has been found,
+ * then "." string is returned.
+ *
+ * @param[in] path A string containing a file path. The behaviour
+ * is undefined if the string contains a path to a directoy. MUST be non-NULL.
+ * @return An allocated string containing the directory name.
+ *
+ * @warning This funciton isn't equivalent to libgen.h dirname().
  */
 BCTBX_PUBLIC char *bctbx_dirname(const char *path);
 
 /**
- * Portable version of the basename function from libgen.h
- * @param[in] path The full path for which we want to find the basename
- * @return NULL if no basename is found, otherwise a copy of the basename of path that needs to be freed with bctbx_free().
+ * @brief Return the name of a file from its path.
+ *
+ * Find the last delimiting character ('/' or '\') and return a
+ * copy of the substring after. If no delimiter has been found,
+ * then a copy of 'path' is returned.
+ *
+ * @param[in] path A string containing a file path. The behaviour
+ * is undefined if the string contains a path to a directoy. MUST be non-NULL.
+ * @return An allocated string containing the file name.
+ *
+ * @warning This funciton isn't equivalent to libgen.h basename().
  */
 BCTBX_PUBLIC char *bctbx_basename(const char *path);
 
+/**
+ * Tests if a file with given pathname exists. Return 0 if yes, -1 otherwise.
+**/
 BCTBX_PUBLIC int bctbx_file_exist(const char *pathname);
 
+/**
+ * Tests if a directory with given pathname exists. Return 0 if yes, -1 otherwise.
+**/
+BCTBX_PUBLIC bool_t bctbx_directory_exists(const char *pathname);
+
 /**
  * @brief return a timeSpec structure(sec and nsec) containing current time(WARNING: there is no guarantees it is UTC ).
  *        The time returned may refers to UTC or last boot.
@@ -436,6 +574,12 @@ BCTBX_PUBLIC void bctbx_timespec_add(bctoolboxTimeSpec *ts, const int64_t lap);
  */
 BCTBX_PUBLIC uint32_t bctbx_time_string_to_sec(const char *timeString);
 
+
+/**
+ * Generate a non cryptographically usable alea
+ *
+ * @return a 32 bits unsigned random. DO NOT USE THIS RANDOM SOURCE FOR ANY CRYPTOGRAPHIC OPERATION
+ */
 BCTBX_PUBLIC unsigned int bctbx_random(void);
 
 
@@ -455,6 +599,14 @@ BCTBX_PUBLIC int bctbx_addrinfo_to_printable_ip_address(const struct addrinfo *a
 BCTBX_PUBLIC int bctbx_sockaddr_to_ip_address(const struct sockaddr *sa, socklen_t salen, char *ip, size_t ip_size, int *port);
 BCTBX_PUBLIC int bctbx_sockaddr_to_printable_ip_address(struct sockaddr *sa, socklen_t salen, char *printable_ip, size_t printable_ip_size);
 
+/** Sort a list of addrinfo with the following rules:
+ -IPV6 including NAT64.
+ -V4 MAPPED IPV6.
+ -V4.
+**/
+BCTBX_PUBLIC struct addrinfo *bctbx_addrinfo_sort(struct addrinfo *ai);
+	
+
 /**
  * Convert a numeric ip address and port into an addrinfo, whose family will be as specified in the first argument.
  * If AF_INET6 is requested, the returned addrinfo will always be an IPv6 address, possibly V4MAPPED if the
@@ -599,7 +751,7 @@ BCTBX_PUBLIC void bctbx_uint32_to_str(uint8_t output_string[9], uint32_t input_u
  *
  * Note : there is no check on the length or validity as an hexa string on the input, incorrect byte is silently mapped to 0
  */
-BCTBX_PUBLIC uint32_t bctbx_str_to_uint32(const uint8_t input_string[9]);
+BCTBX_PUBLIC uint32_t bctbx_str_to_uint32(const uint8_t *input_string);
 
 /**
  * @brief Convert an unsigned 64 bits integer into the corresponding hexadecimal string(including null termination character)
@@ -650,3 +802,8 @@ BCTBX_PUBLIC uint64_t bctbx_str_to_uint64(const uint8_t input_string[17]);
 #else
   #define FORMAT_SIZE_T    "%zu"
 #endif
+
+#if defined(__ANDROID__)
+int mblen(const char* s, size_t n);
+int wctomb(char *s, wchar_t wc);
+#endif
