@@ -42,3 +42,9 @@ char *bctbx_convert_any_to_utf8 (const char *str, const char *encoding) {
 	// TODO change this part when the NDK will contain a usable iconv
 	return bctbx_strdup(str);
 }
+
+wchar_t* bctbx_string_to_wide_string(const char* str){
+// TODO	
+	bctbx_error("Conversion from string to wide string is not implemented");
+    return NULL;
+}
