@@ -189,7 +189,7 @@ int32_t bctbx_signing_key_parse_file(bctbx_signing_key_t *key, const char *path,
 
 
 /*** Certificate ***/
-char *bctbx_x509_certificates_chain_get_pem(bctbx_x509_certificate_t *cert) {
+char *bctbx_x509_certificates_chain_get_pem(const bctbx_x509_certificate_t *cert) {
 	char *pem_certificate = NULL;
 	size_t olen=0;
 
